# -*- coding: utf-8 -*-

import googsystray
import threading, thread
import urllib
import urllib2
import gtk
import time
import sys
import shlex
import os
import subprocess
import webbrowser
import socket
import pprint

import GMain

def dialog_response(widget, res, data=None):
	widget.destroy()


def error_dialog(str):
	diag = gtk.MessageDialog(None, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK,
	                                           _("Error"))
	diag.format_secondary_markup(str)
	diag.connect("response", dialog_response, None)
	diag.show_all()



def run_command(cmd):
	try:
		
		if os.name == "nt":
			subprocess.Popen(cmd)
		else:
			cmd_arr1 = shlex.split(cmd)
			subprocess.Popen(cmd_arr1)
	except OSError:
		error_dialog(_("Could not run command \'%(cmd)s\'") % {"cmd": cmd})




class GBase:
	def check_notifications(self, data=None):
		return False
		
	def get_gconfig(self):
		return GMain._GConf.gconfig["global"]

	def get_gbackupconfig(self):
		return GMain._GConf.backupconfig["global"]

	def check_thr(self, data=None):
		# Yes, these are references, but check_data will never modify
		# the account, so it's OK
		googsystray.debug("Started Thread '" + self.config + "'", 0)

		timeout = float(self.timeout) / 1000.0
		self.thread_cond.acquire()
		while self.thread_run:
			self.check_data()
			if not self.thread_run:
				break
			self.thread_cond.wait(timeout)

		
		self.thread_cond.release()
		googsystray.debug("Ending Thread '" + self.config + "'", 0)

		
	def check_now(self):
		if self.thread_cond.acquire(0):
			self.thread_cond.notify()
			self.thread_cond.release()
		
		
	def stop_timer(self):
		if (self.thread != None):
			self.thread_run = False
			self.thread_cond.acquire()
			self.thread_cond.notify()
			self.thread_cond.release()
			while self.thread.is_alive():
				while gtk.events_pending():
					gtk.main_iteration(False)
			self.thread = None

	def start_timer(self):
		self.stop_timer()
		self.thread_cond.acquire()
		self.thread = threading.Thread(target=self.check_thr)
		self.thread_run = True
		self.thread.start()
		self.thread_cond.release()
		
	def urlopen2(self, account, url, params=None, headers=None, post_data=None, verbose=False, returl=False):
		data = None
		retry_count = 5
		f = None
		real_url = url
		debug_level = 2

		if verbose:
			debug_level = 0

		googsystray.debug("URL is: " + str(url), debug_level)
		googsystray.debug("Params is: " + str(params), debug_level)
		googsystray.debug("Headers is: " + str(headers), debug_level)
		googsystray.debug("Data is: " + str(post_data), debug_level)
		
		while True:
			try:
				if headers == None and post_data == None:
					f = account["opener"].open(url, params)
				else:
					req = urllib2.Request(url, params, headers)
					if post_data is not None:
						req.add_data(post_data)
					f = account["opener"].open(req)
				data = f.read()
				real_url = f.geturl()
				
				if verbose:
					googsystray.debug(f.info())
					googsystray.debug(f.geturl())

				break
			except urllib2.HTTPError, e:
				# HTTP errors, usually permissions or minor
				# google hiccups.
				googsystray.error("HTTPError accessing: " + url + " - " + str(e.code) + " - " + e.msg)
				sys.exc_clear()
				if e.code == 500: # Internal Server Error
					# We see this from Google on occasion.
					# It seems to resolve itself fairly
					# quickly.
					f = None
					req = None
					time.sleep(1.0)
					googsystray.debug(headers)
					googsystray.debug(params)
					continue
				data = None
				break
			except socket.timeout, e:
				googsystray.debug("Socket timeout!")
				# try again
			except urllib2.URLError, e:
				# IO errors.  Network down, typically.  
				# Sleep + count retry.
				googsystray.error("URLError accessing: " + url)
				googsystray.error(e)
				retry_count -= 1
				if retry_count == 0:
					data = None
					break
				time.sleep(5.0)
				
				# Fixme -- we should note on the main logo 
				# perhaps, that we cannot get to one of our sites
					
				continue
			except Exception, e:
				googsystray.error("Unknown Error accessing: " + url)
				sys.exc_clear() 
				data = None
				break
				# We'll try again here
	
		if f is not None:
			# Hack.  Urllib2 leaks memory otherwise.
			if f.fp is not None and hasattr(f.fp, "_sock"):
				f.fp._sock.recv = None
#			else:
#				print "No fp"
#			pprint.pprint(f.fp._sock)

			f.close()
		
		if returl:
			return (data, real_url)
		return data


		
	def _send_post_block(self, args):
		args["lock"].acquire()
		acct = args["account"]
		while True:
			try:
				if args["headers"] == None:
					f = acct["opener"].open(args["url"], args["params"])
				else:
					req = urllib2.Request(args["url"], args["params"],
						args["headers"])
					if args["data"] is not None:
						req.add_data(args["data"])
						
					f = acct["opener"].open(req)
				args["data"] = f.read()
				break
			except urllib2.HTTPError, e:
				googsystray.error("HTTPError accessing: " + args["url"] + " - " + str(e.code) + " - " + e.msg)
				sys.exc_clear()
				args["data"] = None
				break
			except socket.timeout, e:
				googsystray.error("Socket timeout!")
				# try again
			except:
				googsystray.error("Unknown Error accessing: " + args["url"])
				sys.exc_clear() 
				# We'll try again here
		args["event"].set()
		args["lock"].release()
	
	
	def send_post_block(self, account, url, params=None, headers=None, post_data=None):
		event = threading.Event()
		lock = threading.Lock()
		args=[{"url":url,
		       "params":params,
		       "event":event,
		       "lock":lock,
		       "headers":headers,
		       "account":account,
		       "data":post_data,
		       "fp":None}];
		args[0]["lock"].acquire()
		threading.Thread(target=self._send_post_block, args=args).start()
		args[0]["lock"].release()
		
		while not args[0]["event"].isSet():
			while gtk.events_pending():
				gtk.main_iteration(False)
			time.sleep(0.01)
		args[0]["lock"].acquire()
		return args[0]["data"]
		
	def _browser_open(self, url):
		googsystray.debug("Opening: '" + url + "'")
		if self.get_gconfig()["browser_action"] == googsystray.ONCLICK_OTHER:
			try:
				run_command(self.get_gconfig()["browser_cmd"] % url)
			except TypeError, e:
				run_command(self.get_gconfig()["browser_cmd"])
		else:
			webbrowser.open(url, self.get_gconfig()["browser_action"])

	def browser_open(self, url):
		threading.Thread(target=self._browser_open, args=[url]).start()
	

	def get_auth(self, account, service):
		if account["type"] != "Apps":
			type = "GOOGLE"
		else:
			type = "HOSTED"
		
		
		params = urllib.urlencode({'Email':account["username"],
					   'Passwd':account["password"],
					   'accountType':type,
					   'service':service})
		data = self.urlopen2(account, "https://www.google.com/accounts/ClientLogin", params)

		if data == None:
			return
		
		c1 = data.find("Auth=")
		if c1 == -1:
			googsystray.error("Unable to locate authorization for service " + service)
			return
		c2 = data.find("\n", c1)
		if c2 == -1:
			auth = "GoogleLogin auth=" + data[c1+5:]
		else:
			auth = "GoogleLogin auth=" + data[c1+5:c2]
			
		self.auth[account["name"]] = auth

	def clean_label(self, label_str):
		return label_str.replace("&", "&amp;")

	def get_account_from_string(self, name):
		for i in GMain._accounts:
			if i["name"] == name:
				return i
		return None		

	def print_list(self, d, i=0):
		for j in d:
			for k in range(i):
				print "  ",
			if type(j) == types.DictType:
				print "{ "
				self.print_dict(j, i+1)
				for k in range(i):
					print "  ",
				print "}"
			elif type(j) == types.ListType:
				print "["
				self.print_list(j, i+1)
				for k in range(i):
					print "  ",
				print "]"
			elif isinstance(j, basestring):
				print "\"" + str(j) + "\": \"" + str(j) + "\","
			else:
				print str(j) + ","
		
	def print_dict(self, d, i=0):
		for j in d.items():
			for k in range(i):
				print "  ",
			if type(j[1]) == types.DictType:
				print "\"" + j[0] + "\": { "
				self.print_dict(j[1], i+1)
				for k in range(i):
					print "  ",
				print "}"
			elif type(j[1]) == types.ListType:
				print "\"" + j[0] + "\": ["
				self.print_list(j[1], i+1)
				for k in range(i):
					print "  ",
				print "]"
			elif isinstance(j[1], basestring):
				print "\"" + str(j[0]) + "\": \"" + str(j[1]) + "\""
			else:
				print "\"" + j[0] + "\": " + str(j[1])
			
	def dumpit(self, node, depth=0):
		for i in range(depth):
			print "\t",
		try:
			print "<" + node.tag.encode("utf-8"),
			for i in node.attrib:
				print " " + i + " = \"" + node.attrib[i].encode("utf-8") + "\"",
			print ">",
		
			if node.text != None:
				print node.text.encode("utf-8")
			else:
				print ""
		except:
			print "<NOTAG>"
		
		for i in node.getchildren():
			self.dumpit(i, depth+1)


	def __init__ (self):
		self.timeout = googsystray.DEFAULT_TIMEOUT
		self.notification_timer = None
		self.invalidated = False
		self.in_check = False
		self.auth = {}
		
		self.thread = None
		self.thread_run = False
		self.thread_cond = threading.Condition()
		self.thread_lock = threading.RLock()
		



